@@ -262,7 +262,7 @@ class LogixDriver(CIPDriver):
         - *product_code* - code identifying the product type
         - *revision* - dict of {'major': <major rev (int)>, 'minor': <minor rev (int)>}
         - *serial* - hex string of PLC serial number, e.g. ``'FFFFFFFF'``
-        - *device_type* - string value for PLC device type, e.g. ``'1756-L83E/B'``
+        - *product_name* - string value for PLC device type, e.g. ``'1756-L83E/B'``
         - *keyswitch* - string value representing the current keyswitch position, e.g. ``'REMOTE RUN'``
         - *name* - string value of the current PLC program name, e.g. ``'PLCA'``
 
@@ -766,7 +766,7 @@ class LogixDriver(CIPDriver):
         template_name = None
         try:
             for name in (
-                x.decode(errors="replace") for x in data[info_len:].split(b"\x00") if len(x)
+                x.decode(errors="replace") for x in data[info_len:].split(b"\x00")
             ):
                 if template_name is None and ";" in name:
                     template_name, _ = name.split(";", maxsplit=1)
@@ -795,66 +795,30 @@ class LogixDriver(CIPDriver):
 
         _struct_members = []
         _bit_members = {}
-        _host_members = {}  # {offset: (member name, type)}
-        _multibyte_hosts = {}  # hosts for bits that are larger than sints
+        _private_members = set()
+        _unk_member_count = 0
         for member, info in zip(member_names, member_data):
-            if (member.startswith("ZZZZZZZZZZ") or member.startswith("__")) or (
-                predefine and member in {"CTL", "Control"}
+            if not member:  # handle unnamed private members
+                member = f'__unknown{_unk_member_count}'  # double-underscore makes it 'private'
+                _unk_member_count += 1
+            if (
+                member.startswith("ZZZZZZZZZZ") or
+                member.startswith("__") or
+                (predefine and member in {"CTL", "Control"})
             ):
-                _host_members[info["offset"]] = (member, info["type_class"])
-                if info["type_class"].size > USINT.size:
-                    # for host members larger than sints, store what the individual bytes offsets
-                    # would be and use them to update the bit numbers later on for the bool methods
-                    # that way we don't have to mess with changing the offset sizes
-                    # {offset: (member_name, offset in host member, offset in packet)}
-                    _multibyte_hosts.update(
-                        {
-                            info["offset"] + i: (member, i, info["offset"])
-                            for i in range(info["type_class"].size)
-                        }
-                    )
-
+                _private_members.add(member)
             else:
                 data_type["attributes"].append(member)
+
             data_type["internal_tags"][member] = info
 
-            if info["data_type_name"] == "BOOL":
-                if predefine:
-                    if 0 in _host_members and _host_members[0][0] in {"CTL", "Control"}:
-                        # for most predefined types, we assume all 'offsets' refer to the hidden CTL attribute
-                        if info["offset"] in _multibyte_hosts:
-                            _, host_offset, packet_offset = _multibyte_hosts[info["offset"]]
-                            # adjust the bit number by the byte offset within the host member
-                            # but use the packet offset aka host member's offset to map the bit member to the host
-                            _bit_members[member] = (
-                                _host_members[packet_offset][0],
-                                info["bit"] + 8 * host_offset,
-                            )
-                        else:
-                            _bit_members[member] = (_host_members[0][0], info["bit"])
-                    else:
-                        # some predefined types don't list their host members, at least ANALOG_ALARM doesn't
-                        # but if trying to access the whole struct for ANALOG_ALARM leads to a 'permission denied'
-                        # error, so this isn't used anywhere yet and creates a broken StructTag type
-                        # but leaving it in case might find use for it in the future
-                        _bit_members[member] = (info["offset"], info["bit"])
-
-                elif info["offset"] in _host_members:
-                    _bit_members[member] = (_host_members[info["offset"]][0], info["bit"])
-                elif info["offset"] in _multibyte_hosts:
-                    _, host_offset, packet_offset = _multibyte_hosts[info["offset"]]
-                    # adjust the bit number by the byte offset within the host member
-                    # but use the packet offset aka host member's offset to map the bit member to the host
-                    _bit_members[member] = (
-                        _host_members[packet_offset][0],
-                        info["bit"] + 8 * host_offset,
-                    )
-                else:
-                    _struct_members.append((info["type_class"](member), info["offset"]))
+            if info["data_type_name"] == "BOOL" and 'bit' in info:
+                # bit members aren't really 'struct' members since they are aliased to bits of other members
+                _bit_members[member] = (info['offset'], info['bit'])
             else:
                 _struct_members.append((info["type_class"](member), info["offset"]))
 
-        if (
+        if (  # determine if struct is a string or not
             data_type["attributes"] == ["LEN", "DATA"]
             and data_type["internal_tags"]["DATA"]["data_type_name"] == "SINT"
             and data_type["internal_tags"]["DATA"].get("array")
@@ -866,9 +830,9 @@ class LogixDriver(CIPDriver):
             data_type["_struct_members"] = (_struct_members, _bit_members)
             data_type["type_class"] = StructTag(
                 *_struct_members,
-                bool_members=_bit_members,
+                bit_members=_bit_members,
                 struct_size=template["structure_size"],
-                host_members={m: t for m, t in _host_members.values()},
+                private_members=_private_members,
             )
 
         self.__log.debug(f"Completed parsing template as data type {data_type!r}")
@@ -974,7 +938,7 @@ class LogixDriver(CIPDriver):
                             data_type = f"BOOL[{bool_elements}]"
                             result = Tag(request_data["user_tag"], bools, data_type, result.error)
                         else:
-                            val = result.value[bit % 32]
+                            val = result.value[bit]
                             result = Tag(request_data["user_tag"], val, "BOOL", result.error)
                 else:
                     result = Tag(request_data["user_tag"], None, None, result.error)
@@ -1279,6 +1243,8 @@ class LogixDriver(CIPDriver):
 
         """
         base, *attrs = tag_name.split(".")
+        if base.startswith("Program:"):
+            base = f"{base}.{attrs.pop(0)}"
         return self._get_tag_info(base, attrs)
 
     def _get_tag_info(self, base, attrs) -> Optional[dict]:
@@ -1358,12 +1324,9 @@ class LogixDriver(CIPDriver):
                 if idx is not None:
                     tag = f"{_tag}[0]" if rw == "r" else f"{_tag}[{idx // 32}]"
                 bit = idx
-                if implicit_element or elements == 1:
-                    bool_elements = None
-                else:
-                    bool_elements = elements
-                    total_size = (bit or 0) + elements
-                    elements = (total_size // 32) + (1 if total_size % 32 else 0)
+                bool_elements = None if implicit_element or elements == 1 else elements
+                total_size = (bit or 0) + elements
+                elements = (total_size // 32) + (1 if total_size % 32 else 0)
 
             return {
                 "user_tag": request_tag,  # tag name from user, without element request
